'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _PrefixIntervalTree = require('./struct/PrefixIntervalTree');

var _PrefixIntervalTree2 = _interopRequireDefault(_PrefixIntervalTree);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var BUFFER_COLUMNS = 5;
var NO_COLUMNS_SCROLL_RESULT = {
  index: 0,
  offset: 0,
  position: 0,
  contentWidth: 0
};

var FlexiGridHorizontalScrollHelper = function () {
  function FlexiGridHorizontalScrollHelper(columnCount, viewportWidth, getColumnWidth) {
    var _this = this;

    (0, _classCallCheck3.default)(this, FlexiGridHorizontalScrollHelper);

    this.getColumnPosition = function (columnIndex) {
      _this.updateColumnWidth(columnIndex);
      return _this.columnOffsets.sumUntil(columnIndex);
    };

    this.columnCount = columnCount;
    this.viewportWidth = viewportWidth;
    this.getColumnWidth = getColumnWidth;

    this.cachedWidths = [];
    for (var i = 0; i < this.columnCount; i += 1) {
      this.cachedWidths[i] = this.getColumnWidth(i);
    }

    this.columnOffsets = new _PrefixIntervalTree2.default(this.cachedWidths);
    this.contentWidth = this.cachedWidths.reduce(function (memo, width) {
      return memo + width;
    }, 0);
    this.position = 0;

    this.updateWidthsInViewport(0, 0);
  }

  (0, _createClass3.default)(FlexiGridHorizontalScrollHelper, [{
    key: 'setViewportWidth',
    value: function setViewportWidth(viewportWidth) {
      this.viewportWidth = viewportWidth;
    }
  }, {
    key: 'setColumnWidthGetter',
    value: function setColumnWidthGetter(getColumnWidth) {
      this.getColumnWidth = getColumnWidth;
    }
  }, {
    key: 'getContentWidth',
    value: function getContentWidth() {
      return this.contentWidth;
    }
  }, {
    key: 'updateWidthsInViewport',
    value: function updateWidthsInViewport(firstColumnIndex, firstColumnOffset) {
      var width = firstColumnOffset;
      var index = firstColumnIndex;
      while (width <= this.viewportWidth && index < this.columnCount) {
        this.updateColumnWidth(index);
        width += this.cachedWidths[index];
        index += 1;
      }
    }
  }, {
    key: 'updateWidthsLeftViewport',
    value: function updateWidthsLeftViewport(firstColumnIndex) {
      var index = firstColumnIndex - 1;
      while (index >= 0 && index >= firstColumnIndex - BUFFER_COLUMNS) {
        var delta = this.updateColumnWidth(index);
        this.position += delta;
        index -= 1;
      }
    }
  }, {
    key: 'updateColumnWidth',
    value: function updateColumnWidth(columnIndex) {
      if (columnIndex < 0 || columnIndex >= this.columnCount) {
        return 0;
      }

      var oldWidth = this.cachedWidths[columnIndex];
      var newWidth = this.getColumnWidth(columnIndex);
      if (newWidth !== oldWidth) {
        var delta = newWidth - oldWidth;
        this.columnOffsets.set(columnIndex, newWidth);
        this.cachedWidths[columnIndex] = newWidth;
        this.contentWidth += delta;
        return delta;
      }

      return 0;
    }
  }, {
    key: 'getColumnAtEndPosition',
    value: function getColumnAtEndPosition(columnIndex) {
      this.updateColumnWidth(columnIndex);
      var currentColumnIndex = columnIndex;
      var width = this.cachedWidths[currentColumnIndex];
      while (width < this.viewportWidth && currentColumnIndex >= 0) {
        currentColumnIndex -= 1;
        if (currentColumnIndex >= 0) {
          this.updateColumnWidth(currentColumnIndex);
          width += this.cachedWidths[currentColumnIndex];
        }
      }

      var position = this.columnOffsets.sumTo(columnIndex) - this.viewportWidth;
      if (position < 0) {
        position = 0;
      }

      return position;
    }
  }, {
    key: 'scrollTo',
    value: function scrollTo(position) {
      if (this.columnCount === 0) {
        return NO_COLUMNS_SCROLL_RESULT;
      }

      if (position <= 0) {
        // If position less than or equal to 0
        // first column should be fully visible on left.
        this.position = 0;
        this.updateWidthsInViewport(0, 0);

        return {
          index: 0,
          offset: 0,
          position: this.position,
          contentWidth: this.contentWidth
        };
      } else if (position >= this.contentWidth - this.viewportWidth) {
        // If position is equal to or greater than max scroll value, we need
        // to make sure to have bottom border of last column visible.
        var columnIndex = this.columnCount - 1;
        position = this.getColumnAtEndPosition(columnIndex); // eslint-disable-line
      }

      this.position = position;

      var firstColumnIndex = this.columnOffsets.greatestLowerBound(position);
      firstColumnIndex = (0, _utils.clamp)(firstColumnIndex, 0, Math.max(this.columnCount - 1, 0));

      var firstColumnPosition = this.columnOffsets.sumUntil(firstColumnIndex);
      var firstColumnOffset = firstColumnPosition - this.position;

      this.updateWidthsInViewport(firstColumnIndex, firstColumnOffset);
      this.updateWidthsLeftViewport(firstColumnIndex);

      return {
        index: firstColumnIndex,
        offset: firstColumnOffset,
        position: this.position,
        contentWidth: this.contentWidth
      };
    }
  }, {
    key: 'scrollBy',
    value: function scrollBy(delta) {
      if (this.columnCount === 0) {
        return NO_COLUMNS_SCROLL_RESULT;
      }

      var firstColumnIndex = this.columnOffsets.greatestLowerBound(this.position);
      firstColumnIndex = (0, _utils.clamp)(firstColumnIndex, 0, Math.max(this.columnCount - 1, 0));
      var firstColumnPosition = this.columnOffsets.sumUntil(firstColumnIndex);
      var columnIndex = firstColumnIndex;
      var position = this.position;

      var columnWidthChange = this.updateColumnWidth(columnIndex);
      if (firstColumnPosition !== 0) {
        position += columnWidthChange;
      }
      var visibleColumnWidth = this.cachedWidths[columnIndex] - (position - firstColumnPosition);

      if (delta >= 0) {
        while (delta > 0 && columnIndex < this.columnCount) {
          if (delta < visibleColumnWidth) {
            position += delta;
            delta = 0; // eslint-disable-line
          } else {
            delta -= visibleColumnWidth; // eslint-disable-line
            position += visibleColumnWidth;
            columnIndex += 1;
          }
          if (columnIndex < this.columnCount) {
            this.updateColumnWidth(columnIndex);
            visibleColumnWidth = this.cachedWidths[columnIndex];
          }
        }
      } else if (delta < 0) {
        delta = -delta; // eslint-disable-line
        var invisibleColumnWidth = this.cachedWidths[columnIndex] - visibleColumnWidth;

        while (delta > 0 && columnIndex >= 0) {
          if (delta < invisibleColumnWidth) {
            position -= delta;
            delta = 0; // eslint-disable-line
          } else {
            position -= invisibleColumnWidth;
            delta -= invisibleColumnWidth; // eslint-disable-line
            columnIndex -= 1;
          }
          if (columnIndex >= 0) {
            var change = this.updateColumnWidth(columnIndex);
            invisibleColumnWidth = this.cachedWidths[columnIndex];
            position += change;
          }
        }
      }

      var maxPosition = this.contentWidth - this.viewportWidth;
      position = (0, _utils.clamp)(position, 0, maxPosition);
      this.position = position;
      firstColumnIndex = this.columnOffsets.greatestLowerBound(position);
      firstColumnIndex = (0, _utils.clamp)(firstColumnIndex, 0, Math.max(this.columnCount - 1, 0));
      firstColumnPosition = this.columnOffsets.sumUntil(firstColumnIndex);
      var firstColumnOffset = firstColumnPosition - position;

      this.updateWidthsInViewport(firstColumnIndex, firstColumnOffset);
      this.updateWidthsLeftViewport(firstColumnIndex);

      return {
        index: firstColumnIndex,
        offset: firstColumnOffset,
        position: this.position,
        contentWidth: this.contentWidth
      };
    }
  }, {
    key: 'scrollToColumn',
    value: function scrollToColumn(columnIndex) {
      var offset = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 0;

      var targetIndex = (0, _utils.clamp)(columnIndex, 0, Math.max(this.columnCount - 1, 0));
      var targetOffset = (0, _utils.clamp)(offset, -this.cachedWidths[targetIndex], 0);
      var position = this.columnOffsets.sumUntil(targetIndex);
      return this.scrollTo(position - targetOffset);
    }
  }, {
    key: 'scrollColumnIntoView',
    value: function scrollColumnIntoView(columnIndex) {
      var targetIndex = (0, _utils.clamp)(columnIndex, 0, Math.max(this.columnCount - 1, 0));
      this.updateColumnWidth(targetIndex);
      var startPosition = this.columnOffsets.sumUntil(targetIndex);
      var endPosition = startPosition + this.cachedWidths[targetIndex];

      if (startPosition < this.position) {
        return this.scrollTo(startPosition);
      } else if (this.position + this.viewportWidth < endPosition) {
        var position = this.getColumnAtEndPosition(targetIndex);
        return this.scrollTo(position);
      }

      return this.scrollTo(this.position);
    }
  }]);
  return FlexiGridHorizontalScrollHelper;
}();

exports.default = FlexiGridHorizontalScrollHelper;
module.exports = exports['default'];